#include "Adafruit_WS2801.h"
#include "SPI.h" // Comment out this line if using Trinket or Gemma
#ifdef __AVR_ATtiny85__
 #include <avr/power.h>
#endif


const int LEDAmount = 25;

int cells[]={0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0};
int generation =0;
int ruleset[]={0,1,0,1,1,1,0,0};

int flex; 
int timer = 0;
int timeInterval = 5142;
uint8_t dataPin  = 2;    // Yellow wire on Adafruit Pixels
uint8_t clockPin = 5;    // Green wire on Adafruit Pixels
 
Adafruit_WS2801 strip = Adafruit_WS2801(LEDAmount, dataPin, clockPin);

void setup() {

  Serial.begin(4800);
  //Serial.println("work");
  strip.begin();
  strip.show();
  randomSeed(analogRead(A2));
  int randomInt = random(0,255);  
  setNewRule(randomInt);
  Serial.println(randomInt);
}

void loop() {
   delay(700);
   timer += 1;
//   if(timer > timeInterval){
//    int randomInt = random(0,255);  
//    setNewRule(randomInt);
//    //Serial.println(randomInt);
//    timer = 0;
//   }
    int sensorDiff = flex-analogRead(A0);
   if(abs(sensorDiff)>10){
    flex = analogRead(A0);
    int flexMapped = flex%255;
    setNewRule(flexMapped);
    drop(20);
   }   
   generate();
   show();
   Serial.println(analogRead(A0));
   //Serial.println(generation);
}

void generate(){
  int nextgen[LEDAmount] ={};
  for (int i =1; i<LEDAmount-1; i++){
    int left = cells[i-1];
    int middle = cells[i];
    int right = cells [i+1];
    nextgen[i] = rules(left,middle,right);
    //Serial.println(cells[i]);
  }
  for(int i=0;i<LEDAmount; i++){
    cells[i] = nextgen[i];
    //Serial.print(nextgen[i]);
  }
  generation ++;
}

int rules (int a, int b, int c){
  String as= String(a);
  String bs = String(b);
  String cs = String(c);
  String s = String(as + bs + cs);
  char *endPtr;
  int index = strtol(s.c_str(),&endPtr,2);
  return ruleset[index];  
}

void show(){
  for (int i=0; i<LEDAmount; i++){
    uint32_t red = Color(250,0,0);
    uint32_t on = Color(255,255,255);
    uint32_t green = Color(0,250,0); 
    int sum = 0;
    sum += cells[i-1];
    sum += cells[i];
    sum += cells[i+1];
    if(sum == 0){ 
      strip.setPixelColor(i,red);
    }
    else if(sum == 1){
      strip.setPixelColor(i,green);
    }
    else {
      strip.setPixelColor(i,on);
    }
  }
  //strip.setPixelColor(12,Color(250,160,0));
  strip.show();
}

uint32_t Color(byte r, byte g, byte b)
{
  uint32_t c;
  c = r;
  c <<= 8;
  c |= g;
  c <<= 8;
  c |= b;
  return c;
}

void createRuleset(int b){
  const byte numDigs = 8;
  byte num = b; 
  for (byte i=0; i<numDigs; i++){
    ruleset[7-i] = bitRead(num,i);
  }
}

void setNewRule (int val){
  int cells[]={0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0};
  createRuleset(val);
}

void drop(int wait){
  
  int brightnessStep = int(255/LEDAmount);  
  for(int i=0; i<LEDAmount-1; i++){
    allOff(LEDAmount-1);
    int brightness = (i+1)*brightnessStep;
//int brightness = 10;
    uint32_t c = Color(brightness,brightness,0);
    strip.setPixelColor(i,c);
    strip.setPixelColor(LEDAmount-1,Color(255,255,0));
    strip.show();
    delay(wait);
  }
}

void allOff(int amount){
  for(int i=0; i<amount; i++){
    strip.setPixelColor(i,Color(0,0,0));
  }
  strip.show();
}



